#pragma implementation
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <misc.h>
#include <configf.h>
#include <usercomng.h>
#include <userconf.h>
#include <translat.h>
#include <fviews.h>
#include "fetchmailconf.h"
#include "fetchmailconf.m"
#include "serveredit.h"
#include "useridedit.h"
#include "comng.h"

/*
 * Co-manager user class
 */
static HELP_FILE help_fetchmailcomng( "fetchmailconf", "usercomng" );

static PRIVILEGE p_userid ("fetchmailconf_comng"
    ,P_MSG_U(T_PRIVUSERSFETCHMAIL,"Users/Fetchmail")
    ,P_MSG_R(T_PRIVILEGE));


PUBLIC FETCHMAIL_COMNG::FETCHMAIL_COMNG( DICTIONARY &dictionary )
	: USERACCT_COMNG( dictionary )
{
//fprintf( stderr, "comng.cc: FETCHMAIL_COMNG::FETCHMAIL_COMNG()\n");
	is_new = dictionary.get_str( "is_new" );
	local_user = dictionary.get_str( "name" );
}


PUBLIC void FETCHMAIL_COMNG::setupdia( DIALOG &dia )
{
//fprintf( stderr, "comng.cc: FETCHMAIL_COMNG::setupdia\n");
	serverlist.read();
	if ( serverlist.getnb() == 0 ) {
		return;
	}
	if ( local_user.is_empty() ) {
		/*
		 * It appears dictionary is not updated for a new user so
		 * there is no way to get user name: So no fetchmailconf
		 * for new users. If/when this is fixed the logic is in place...
		 */
		return;
	}
	if ( ! perm_checkpriv( &p_userid ) ) {
		return;
	}
	dia.addhelp( help_fetchmailcomng,MSG_U(T_FETCHMAIL_COMNG,"Fetch mail"));
	dia.newf_title( MSG_R(T_FETCHMAIL_COMNG),1,"",MSG_R(T_FETCHMAIL_COMNG));

	for (int i=0; i<serverlist.getnb(); i++) {
		SERVER *server = serverlist.getitem(i);
		const char *server_name = server->server_name.get();

//fprintf( stderr, "comng.cc: server_name=%s\n",server_name);
		dia.newf_title( server_name, 2, "", server_name );
		useridlist.remove_all();
		useridlist.read( server );
		USERID *new_userid;
		USERID *userid = useridlist.getitem( local_user.get() );
		if ( userid == NULL ) {
			new_userid = new USERID( );
			new_userid->local_user.setfrom( local_user );
			new_userid->new_userid = true;
//fprintf( stderr, "comng.cc: NEW: server_name=%s local_user=%s\n",server_name, new_userid->local_user.get());
		} else {
			new_userid = new USERID( );
			new_userid->local_user.setfrom( local_user );
			new_userid->remote_user.setfrom( userid->remote_user );
			new_userid->password.setfrom( userid->password );
			new_userid->new_userid = false;
		}
		new_userid->server = server;
		new_useridlist.add( new_userid );
		dia.newf_str(MSG_R(F_REMOTEUSER), new_userid->remote_user );
		dia.newf_str(MSG_R(F_USERPASSWORD), new_userid->password );
		if ( ! new_userid->new_userid ) {
        		dia.newf_chk ("",new_userid->delete_user,MSG_U(F_DELETE, "Delete this entry"));
		}
	}
}

PUBLIC int FETCHMAIL_COMNG::validate( DIALOG &dia, int &nof )
{
//fprintf( stderr, "comng.cc: FETCHMAIL_COMNG::validate\n");
	// -1 at error
	return( 0 );
}

PUBLIC int FETCHMAIL_COMNG::save( PRIVILEGE *privilege )
{
//fprintf( stderr, "comng.cc: FETCHMAIL_COMNG::save\n");
	SERVER *server = NULL;
	USERID *userid = NULL;
	for ( int s=0; s< serverlist.getnb(); s++ ) {
		server = serverlist.getitem( s );
		userid = new_useridlist.getitem( s );
//fprintf( stderr, "server->server_name=%s userid->remote_user=%s userid->password=%s userid->local_user=%s %s\n",server->server_name.get(), userid->remote_user.get(), userid->password.get(), userid->local_user.get()  userid->new_userid?"new_userid":"old_userid");
		if ( userid->new_userid ) {
			if ( ! userid->remote_user.is_empty() ) {
				userid->write( MENU_ACCEPT );
			}
		} else {
			if ( userid->delete_user ) {
				userid->write( MENU_DEL );
			} else {
				if ( ( ! userid->remote_user.is_empty() )
				&& ( ! userid->password.is_empty() ) ) {
					userid->write( MENU_ACCEPT );
				}
			}
		}
	}
	return( 0 );
}

PUBLIC int FETCHMAIL_COMNG::deluser( PRIVILEGE *privilege )
{
//fprintf( stderr, "comng.cc: FETCHMAIL_COMNG::deluser\n");
	SERVER *server = NULL;
	USERID *userid = NULL;
	for ( int s=0; s< serverlist.getnb(); s++ ) {
		server = serverlist.getitem( s );
		userid = new_useridlist.getitem( s );
//fprintf( stderr, "server->server_name=%s userid->remote_user=%s userid->password=%s %s\n",server->server_name.get(), userid->remote_user.get(), userid->password.get(), userid->new_userid?"new_userid":"old_userid");
		userid->write( MENU_DEL );
	}
	return( 0 );
}
