#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <misc.h>
#include "internal.h"
#include "../paths.h"
#include "samba.m"
#include <userconf.h>
#include <subsys.h>
#include <translat.h>
#include "keyword.h"

SAMBA_HELP_FILE help_samba ("samba");

static LINUXCONF_SUBSYS subb (subsys_samba,P_MSG_R(M_SAMBA));

CONFIG_FILE smb_conf (ETC_SMB_CONF,help_samba
		,CONFIGF_MANAGED
		,subsys_samba);

static PRIVILEGE p_samba ("samba"
	,P_MSG_U(T_PRIVISAMBA,"Samba administration")
	,P_MSG_U(T_SERVICES,"1-Services"));

/* #Specification: samba config / principle
	Linuxconf supports the samba file server.
*/

static const char K_SAMBA[]="samba";
static const char K_SYNCPASS[]="syncpass";

PUBLIC SMB_ITEM::SMB_ITEM ()
{
}

PUBLIC SMB_ITEM::SMB_ITEM (const char *_comment, const char *line)
{
	comment.setfrom (_comment);
	char buf[100];
	char *pt = buf;
	line = str_skip(line);
	while (1){
		while (*line > ' ' && *line != '=') *pt++ = *line++;
		line = str_skip (line);
		if (*line == '='){
			line = str_skip (line+1);
			val.setfrom (line);
			break;
		}else if (*line == '\0'){
			break;
		}else{
			// We add a single space
			*pt++ = ' ';
		}
	}
	*pt = '\0';
	key.setfrom (buf);
}

PUBLIC void SMB_ITEM::write (FILE *fout) const
{
	fputs (comment.get(),fout);
	if (val.is_empty()){
		fprintf (fout,"    %s\n",key.get());
	}else{
		fprintf (fout,"    %s = %s\n",key.get(),val.get());
	}
}

PUBLIC SMB_ITEM *SMB_ITEMS::getitem(int no) const
{
	return (SMB_ITEM*)ARRAY::getitem(no);
}

PUBLIC SMB_ITEM *SMB_ITEMS::getitem(const char *key) const
{
	SMB_ITEM *ret = NULL;
	int n = getnb();
	for (int i=0; i<n; i++){
		SMB_ITEM *it = getitem(i);
		if (it->key.cmp(key)==0){
			ret = it;
			break;
		}
	}
	return ret;
}


PUBLIC const char *SMB_ITEMS::getval(const char *key) const
{
	SMB_ITEM *it = getitem(key);
	if (it == NULL && strcmp (key,K_PUBLIC)==0){
		it = getitem("guest ok");
	}
	return it == NULL ? "" : it->val.get();
}

PUBLIC const char *SMB_ITEMS::getval(
	const char *key1,
	const char *key2,
	const char *key3) const
{
	char buf[100];
	char *pt = stpcpy (buf,key1);
	if (key2 != NULL){
		*pt++ = ' ';
		pt = stpcpy (pt,key2);
		if (key3 != NULL){
			*pt++ = ' ';
			strcpy (pt,key3);
		}
	}
	return getval (buf);
}

PUBLIC void SMB_ITEMS::setval(const char *key,const char *val)
{
	SMB_ITEM *it = getitem(key);
	if (it == NULL && strcmp (key,K_PUBLIC)==0){
		it = getitem("guest ok");
	}
	if (val == NULL || val[0] == '\0'){
		remove_del (it);
	}else{
		if (it == NULL){
			it = new SMB_ITEM;
			add (it);
		}
		it->key.setfrom (key);
		it->val.setfrom (val);
	}
}

PUBLIC void SMB_ITEMS::setval(const char *key,const SSTRING &val)
{
	setval (key,val.get());
}

PUBLIC void SMB_ITEMS::setval(const char *key, int num)
{
	char buf[20];
	sprintf (buf,"%d",num);
	setval (key,buf);
}

PUBLIC void SMB_ITEMS::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	const char *val)
{
	char buf[100];
	char *pt = stpcpy (buf,key1);
	if (key2 != NULL){
		*pt++ = ' ';
		pt = stpcpy (pt,key2);
		if (key3 != NULL){
			*pt++ = ' ';
			strcpy (pt,key3);
		}
	}
	setval (buf,val);
}

PUBLIC void SMB_ITEMS::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	const SSTRING &val)
{
	setval (key1,key2,key3,val.get());
}
PUBLIC void SMB_ITEMS::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	int num)
{
	char buf[20];
	sprintf (buf,"%d",num);
	setval (key1,key2,key3,buf);
}
PUBLIC void SMB_ITEMS::write(FILE *fout) const
{
	int n = getnb();
	for (int i=0; i<n; i++){
		getitem(i)->write (fout);
	}
}

PUBLIC SMB_SHARE::SMB_SHARE(const char *_comment, const char *_name)
{
	comment.setfrom (_comment);
	name.setfrom (_name);
}
PUBLIC SMB_SHARE::SMB_SHARE()
{
}

PUBLIC const char *SMB_SHARE::getval(const char *key) const
{
	return items.getval(key);
}

PUBLIC const char *SMB_SHARE::getval(
	const char *key1,
	const char *key2,
	const char *key3) const
{
	return items.getval (key1,key2,key3);
}

PUBLIC void SMB_SHARE::setval(const char *key, const SSTRING &val)
{
	items.setval(key,val);
}
PUBLIC void SMB_SHARE::setval(const char *key, int num)
{
	items.setval(key,num);
}

PUBLIC void SMB_SHARE::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	const SSTRING &val)
{
	items.setval (key1,key2,key3,val);
}
PUBLIC void SMB_SHARE::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	int num)
{
	items.setval (key1,key2,key3,num);
}
PUBLIC void SMB_SHARE::setval(
	const char *key1,
	const char *key2,
	const char *key3,
	const char *val)
{
	items.setval (key1,key2,key3,val);
}

PUBLIC void SMB_SHARE::write (FILE *fout) const
{
	fputs (comment.get(),fout);
	fprintf (fout,"[%s]\n",name.get());
	items.write (fout);
}

PUBLIC SMB_SHARE *SMB_SHARES::getitem(int no) const
{
	return (SMB_SHARE *)ARRAY::getitem(no);
}

PUBLIC SMB_SHARE *SMB_SHARES::getitem(const char *name) const
{
	SMB_SHARE *ret = NULL;
	int n = getnb();
	for (int i=0; i<n; i++){
		SMB_SHARE *d = getitem(i);
		if (d->name.cmp(name)==0){
			ret = d;
			break;
		}
	}
	return ret;
}

/*
	Read lines and accumulate comments.
	buf will contain the first non comment, non empty line.
*/
static const char *fgets_samba (
	char buf[],
	int size,
	FILE *fin,
	SSTRING &comments)
{
	comments.setfrom ("");
	const char *ret = NULL;
	while (fgets_cont(buf,size,fin)!=-1){
		strip_end (buf);
		char *pt = str_skip (buf);
		if (pt[0] == '\0' || pt[0] == '#' || pt[0] == ';'){
			strcat (buf,"\n");
			comments.append (pt);
		}else{
			ret = buf;
			break;
		}
	}
	return ret;
}


PUBLIC SMB_CONF::SMB_CONF ()
{
	syncpass = linuxconf_getvalnum (K_SAMBA,K_SYNCPASS,1);
	FILE *fin = smb_conf.fopen ("r");
	if (fin != NULL){
		char buf[3000];
		SSTRING comments;
		/* #Specification: smb.conf / parsing
			The [global] share is optional. Linuxconf assumes that
			anything at the start of the /etc/smb.conf is part of the
			global section. If the [global] section is reached, linuxconf
			just continue over it. The [global] section will be written
			back though, even if it was missing.

			Note that Linuxconf tries to cope with the following
			sequences

			#
			---------
			some comments
			[global]
			directives
			---------

			---------
			some comments
			directive
			[global]
			directive
			-------------

			----------------
			comments
			directives
			[other section]
			-----------------

			Linuxconf expects comments starting with # or ;.
		*/
		SMB_SHARE *global = NULL;
		SMB_ITEMS *ptitems = NULL;
		while (fgets_samba(buf,sizeof(buf)-1,fin,comments)!=NULL){
			char *pt = str_skip (buf);
			if (*pt == '['){
				pt = str_skip(pt+1);
				char *end = strchr(pt,']');
				if (end != NULL) *end = '\0';
				strip_end (pt);
				if (strcmp(pt,"global")==0 && global != NULL){
					ptitems = &global->items;
				}else{
					SMB_SHARE *share = new SMB_SHARE(comments.get(),pt);
					shares.add (share);
					ptitems = &share->items;
					if (strcmp(pt,"global")==0) global = share;
				}
			}else{
				if (ptitems == NULL){
					global = new SMB_SHARE("","global");
					shares.add (global);
					ptitems = &global->items;
				}
				ptitems->add (new SMB_ITEM(comments.get(),buf));
			}
		}
		fclose (fin);
	}			
}


PUBLIC int SMB_CONF::write ()
{
	int ret = -1;
	FILE *fout = smb_conf.fopen (&p_samba,"w");
	if (fout != NULL){
		linuxconf_setcursys (subsys_samba);
		int i;
		for (i=0; i<shares.getnb(); i++){
			shares.getitem(i)->write(fout);
		}
		ret = fclose (fout);
		linuxconf_replace (K_SAMBA,K_SYNCPASS,syncpass);
		linuxconf_save();
	}
	return ret;
}




